use <mcad/involute_gears.scad>;

/* INPUT MOTOR */
im_screw=1.45 * 25.4 * cos(45); // NEMA14 screw spacing
im_loc=22+0.5;				// locating circle on motor
im_bore =3;                     // M3 diameter
im_head =6;                     // 6mm bolt heads for m3
im_shaft=5;                     // 5mm input shaft

/* OUTPUT MOTOR */
om_screw=1.72 * 25.4 * cos(45); // output motor screw spacing
om_bore=3;
om_head=6;
om_shaft=5;

/* GEAR SETTINGS */
ipitch = 20;    // imperial pitch
pitch=ipitch / 25.4;    // metric pitch?
gearh=8;        // thickness of gears
ring=24;        // number of teeth on ring
sun=8;          // teeth on sun
planet=8;       // teeth on planet

/* MISCELLANEOUS SETTINGS */
towerh=30-(4+3);    // how tall the tower section is   
t=1;    // Minimum thickness of plastic
$fa=1.5;
$fs=1;

/* These are all the parts this file can generate.  Uncomment what you want. */

/* Sun gear.  sun(shaft_diameter,flat);
   flat=0 means no flat, flat=1 cuts 1mm off the radius, etc. */
sun(4 + 0.5, 0.55);	 // My best guess to fit a 4mm shaft, untested

//for(N=[0:2]) translate(N*[15,0]) planet();
//carrier_top();
//carrier_bottom();
//orthanc();
//testplate();      /* NEMA14 - NEMA17 adaptor plate. */


module squarepat(W) for(A=[0:90:360-1]) rotate([0,0,A]) translate(W*[1,1]/2) children();
module crosspat(W)	for(A=[0,90]) rotate([0,0,A]) hull() { translate(W*[1,1]/2) children(); rotate([0,0,180]) translate(W*[1,1]/2) children(); }
module snap(l) {	translate([l/pitch,0]) children(); }

module orthanc(h=towerh, h2=12, a=20) difference() {
    
    union() {
        linear_extrude(h) hull() squarepat(im_screw) circle(d=im_head+t+t);
        
        hull() {
            rotate([0,0,a]) translate([0,0,h-h2]) {
                linear_extrude(h2) hull() squarepat(om_screw) circle(d=im_head+t+t);
            }
            
            cylinder(d=10, h=1);
        }
    }
    // Input motor screws
    squarepat(im_screw) translate([0,0,-1]) cylinder(d=im_bore, h=h+2);
    
    for(A=[0,90]) rotate([0,0,A])
    hull() for(U=[-1,1]) translate(U*im_screw/2 * [1,1,0])
        translate([0,0,h+1]) rotate([180,0]) cylinder(d=im_head, h=1+4);
    
    // Output motor screws
    translate([0,0,h-h2]) rotate([0,0,a]) squarepat(om_screw)
	{
       cylinder(d=om_bore, h=h2+2);
		translate([0,0,h2/2]) linear_extrude(2.75,center=true) hull() {
			for(Y=[0,10]) translate([0,Y])rotate([0,0,30])  circle(d=5.5/cos(30)+1, $fn=6);
		}
	}

	// Ring gear
    translate([0,0,2]) rotate([0,0,180]/ring) mygear(ring, h+2,inside=1);

	translate([0,0,-1]) cylinder(d=22+0.5, h=5);

//	translate([0,0,-1]) cylinder(d=im_loc, h=h+2);
}

module planets() for(A=[-45,45,180]) rotate([0,0,A]) snap((sun+planet)/2) children();

module carrier(h=gearh+5+5+1) 
intersection() {
	union() {
		difference() { 
			linear_extrude(5, convexity=3) difference() {
				circle(d=(ring/pitch)-3);
				circle(d=4+(sun/pitch));
				planets() circle(d=3);
			}

			translate([0,0,-1]) planets() cylinder(d=5.5/cos(30)+1, $fn=6, h=4);
		}

		translate([0,0,4]) linear_extrude(gearh+3, convexity=3) 
		offset(1) offset(-1) difference() {
			circle(d=(ring/pitch)-3);
			circle(d=4+(sun/pitch));
			planets() circle(d=(planet/pitch)+3.5);
		}

		translate([0,0,5+gearh+0.5]) difference() { union() {
				linear_extrude(2,convexity=3) difference() {
					circle(d=(ring/pitch)-3);	
					planets() circle(d=3);
					circle(d=5.5);
				}

				translate([0,0,1]) linear_extrude(4) offset(1) offset(-1) difference() {
					circle(d=(ring/pitch)-3);
					circle(d=5);

					planets() hull() {
						circle(d=7);
						translate([5,0]) circle(d=7);
					}
				}
			}
			translate([0,0,-1]) rotate([0,0,30]) cylinder(d=8/cos(30)  + 0.4, h=5, $fn=6);
		}
	}
}

dots=[for(a=[0,110],A=a+90) 12*[sin(A),cos(A)] ];
ch=13.5;

module carrier_top() {
	translate([0,0,5]) rotate([180,0,0]) split(ch,top=1,caves=dots)
		carrier();
}


module carrier_bottom() {
	translate([0,0,ch]) split(ch,top=0,marks=dots)
			carrier();
}

module planet() {
	difference() {
		mygear(planet);
		cylinder(d=3+0.75, h=50, center=true);
		// lop corners off top and bottom of teeth
		for(w=4.5) rotate_extrude() polygon([[w,-1], [w+5,4], [w,9], [20,9], [20,-1]]);
	}
}

module sun(d=5.75, flat=0.5) difference() {
	union() {
		mygear(sun,gearh+7);
		cylinder(d=21.5, h=2);
//		cylinder(d=12.75, 5);
	}
	translate([0,0,-1]) linear_extrude(gearh+5+2+2) difference() {
		circle(d=d);
		if(flat) {
			translate([d,0]/2) square([flat*2, d+2],center=true);
		}
	}
	// lop corners off top of teeth
	for(w=4)translate([0,0,7]) rotate_extrude() polygon([[w+5,5], [w,9], [20,9] ]);
}

//gt_gear(sun) for(A=[-55,55,180]) rotate([0,0,A]) gt_gear(planet,ao=A);

module testplate(a=-20) union() {
	difference() {
		linear_extrude(5, convexity=3) difference() {
			union() {
				crosspat(im_screw) circle(d=10);
				rotate([0,0,a]) crosspat(om_screw) circle(d=10);
			}
			squarepat(im_screw) circle(d=3.5);
			rotate([0,0,a]) squarepat(om_screw) circle(d=3.5);
			circle(d=7);
		}
		//translate([0,0,2]) rotate([0,0,20]) squarepat(om_screw) cylinder(d=6, h=10);
		translate([0,0,-1]) squarepat(im_screw) cylinder(d=5.5/cos(30)+0.75, $fn=6, h=3+1);
		translate([0,0,-1]) cylinder(d=23+0.75, h=3);
	}

	translate([0,0,2]) linear_extrude(5) difference() {
		circle(d=22);
		circle(d=7);
	}

	rotate([0,0,a]) squarepat(om_screw) linear_extrude(8) difference() {
		circle(d=6);
		circle(d=3.5);
	}
}

module mygear(teeth, t=gearh, inside=0, backlash=0.45) {
    // Finicky Gear Settings
    //backlash=0.45;
    addendum_adjustment=1.4;
    pressure_angle=20; 
    
    if(inside)
		mygear(teeth,t,inside=0,backlash=-backlash);
    else {
        gear(
            hub_diameter=0, bore_diameter=0, clearance=0,
            number_of_teeth=teeth, diametral_pitch=pitch, rim_thickness=t,
            gear_thickness=t, backlash=backlash,
            pressure_angle=pressure_angle, addendum_adjustment=addendum_adjustment);
    }
}

// Cuts something in half and adds marks to put them back together
module split(z=10, marks=[], caves=[],top=0) {
	difference() {
		union() {
			if(top) {
				intersection() {
					translate([0,0,-z]) children();
					linear_extrude(500) square([500,500],center=true);
				}
			}
			else
			{
				difference() {
					translate([0,0,-z]) children();
					linear_extrude(500) square([500,500],center=true);
				}
			}

			for(U=marks) translate(U) sphere(d=3);
		}
		for(U=caves) translate(U) sphere(d=3.5);
	}
}
